#include <windows.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <process.h>
#include "cmdline.h"
#include "types.h"

/* Globals (for settings that apply to the whole program) */
struct gengetopt_args_info options;

void perror_message(uint32 error)
{
    char temp[300];

    FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM,
                   NULL, error, 0, temp, 300, NULL);
    puts(temp);
}

int main (int argc, char *argv[])
{
    HANDLE ser_handle = NULL;
    DCB ser_dcb;
    DWORD modem_status;
    DWORD event;

    /* Parse the command line for options to use */
    if(cmdline_parser(argc, argv, &options) != 0)
	return 1;

    if(options.execute_given) {
	if(strchr(options.execute_arg, ' ') != NULL) {
	    if(!options.quiet_flag) {
		printf("Sorry, spaces in execute statements aren't supported (yet)\n");
		printf("Make a .bat file instead with all the arguments\n");
	    }
	    exit(0);
	}
    }

    /* Open serial port */
    ser_handle = CreateFile(options.port_arg,
			   GENERIC_READ | GENERIC_WRITE,
			   0,
			   NULL,
			   OPEN_EXISTING,
			   FILE_ATTRIBUTE_NORMAL,
			   NULL);
    if (ser_handle==INVALID_HANDLE_VALUE) {
	if(!options.quiet_flag)
	    printf("\nCan't open serial port %s\n", options.port_arg);
	return 2;
    }

    /* Set RTS line positive so that we can have something
       well defined to connect the other lines to */
    GetCommState(ser_handle, &ser_dcb);
    ser_dcb.fOutxCtsFlow = TRUE;
    ser_dcb.fOutxDsrFlow = TRUE;
    ser_dcb.fRtsControl = RTS_CONTROL_ENABLE;
    SetCommState(ser_handle, &ser_dcb);

    /* Set mask to only act on the right kind of events */
    if(SetCommMask(ser_handle, EV_CTS | EV_DSR) == 0) {
	if(!options.quiet_flag)
	    printf("Can't set event mask\n");
	exit(2);
    }

    if(GetCommModemStatus(ser_handle, &modem_status) == 0) {
	if(!options.quiet_flag)
	    printf("Can't get modem status from com port\n");
	exit(2);
    }
    if(options.verbose_flag) {
	printf("CTS: %-3s  ", ((MS_CTS_ON & modem_status) != 0) ? "on" : "off");
	printf("DSR: %s\n", ((MS_DSR_ON & modem_status) != 0) ? "on" : "off");
    }

    while(TRUE) {
	/* Wait for something useful to happen */
	if(WaitCommEvent(ser_handle, &event, NULL) == 0) {
	    if(!options.quiet_flag) {
		printf("Something wrong with WaitCommEvent();\n");
		printf("(This shouldn't happen, you know...)\n");
	    }
	    exit(2);
	}

	if(((event & EV_CTS) != 0) || ((event & EV_DSR) != 0)) {
	    /* A wanted event arrived */
	    if(GetCommModemStatus(ser_handle, &modem_status) == 0) {
		if(!options.quiet_flag)
		    printf("Can't get modem status from com port\n");
		exit(2);
	    }
	    if(options.verbose_flag) {
		printf("CTS: %-3s  ",
		       ((modem_status & MS_CTS_ON) != 0) ? "on" : "off");
		printf("DSR: %s\n",
		       ((modem_status & MS_DSR_ON) != 0) ? "on" : "off");
	    }
	    if(((modem_status & MS_CTS_ON) && (options.cts_flag)) ||
	       ((modem_status & MS_DSR_ON) && (options.dsr_flag))) {
		/* Seems OK to do the command now */
		if(options.execute_given) {
		    if(!options.quiet_flag) {
			if(_spawnl(_P_NOWAIT, options.execute_arg, NULL) == -1)
			    printf("Process wasn't spawned properly.\n");
		    } else {
			_spawnl(_P_DETACH, options.execute_arg, NULL);
		    }
		} else {
		    if(!options.quiet_flag)
			printf("Nothing to do...\n");
		}
	    }
	}

	/* For safety purposes, so it doesn't spawn totally uncontrollably */
	Sleep(500);
    }

    /* Yes, I know we never get here, but it feels better... */
    CloseHandle(ser_handle);
    return 0;
}
